/*
* IceBreaker
* Copyright (c) 2000-2002 Matthew Miller <mattdm@mattdm.org> and
*   Enrico Tassi <gareuselesinge@infinito.it>
*
* <http://www.mattdm.org/icebreaker/>
*
* This program is free software; you can redistribute it and/or modify it
* under the terms of the GNU General Public License as published by the Free
* Software Foundation; either version 2 of the License, or (at your option)
* any later version.
*
* This program is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
* or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along
* with this program; if not, write to the Free Software Foundation, Inc., 59
* Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
*/

#include <SDL.h>
#include <stdio.h>
#include <string.h>
#include "icebreaker.h"
#include "globals.h"
#include "text.h"
#include "laundry.h"
#include "dialog.h"
#include "sound.h"
#include "hiscore.h"
#include "options.h"
#include "fullscreen.h"
#include "themes.h"
#include "grid.h"
#include "status.h"
#include "level.h"
#include "event.h"

// fix -- this file is too big; we should split the menu stuff out
// and put it in menu.c

static int menuhandler(SDL_Rect menurect, int menulength, char ** menuitems, int (**menufunctions)(char *), int menuvaluetextwidth );
static void drawmenu(SDL_Rect menurect, int menulength, char ** menuitems, int (**menufunctions)(char *), int menuvaluetextwidth, int highlighted);

static int menuitem_newgame(char * notused);
static int menuitem_options(char * notused);
static int menuitem_highscores(char * notused);
static int menuitem_help(char * notused);
static int menuitem_quit(char * notused);

static int menuitem_sound(char * val);
static int menuitem_autopause(char * val);
static int menuitem_fullscreen(char * val);
static int menuitem_difficulty(char * val);
static int menuitem_theme(char * val);

static int yesnodialog(char* text1, char* text2, char* text3);


// puting this here is ugly, but it's less ugly than many of the
// alternatives -- it's very hard to switch themes on the fly
static SDL_Surface * optionsmenusave;

#define MAXMENUITEMS 10
#define MAXMENUVALUELENGTH 10

int gethighusername(int highest)
{
	SDL_Rect tmprect,namerect,okayrect;
	SDL_Event event;
	int done=false;
	int quit=false;
	int okayglow=false;
	int okaypressed=false;
	int clearname=true;
	
	char keypressed;
	int insertionpoint=0;
	
	tmprect.w=282;
	tmprect.h=92;
	tmprect.x=(WIDTH/2)-(tmprect.w/2)-1;
	tmprect.y=(HEIGHT/2)+31;
	SDL_FillRect(screen,&tmprect,color.gridline);

	tmprect.w-=2; tmprect.h-=2;
	tmprect.x++; tmprect.y++;
	SDL_FillRect(screen,&tmprect,color.background);

	if (highest)
		puttext(tmprect.x+7,tmprect.y+7,2,color.normaltext,"NEW TOP SCORE!");
	else
		puttext(tmprect.x+7,tmprect.y+7,2,color.normaltext,"NEW HIGH SCORE.");
	puttext(tmprect.x+7,tmprect.y+7+16,2,color.normaltext,"ENTER YOUR NAME:");
	
	namerect.w=268;
	namerect.h=29;
	namerect.x=tmprect.x+6;
	namerect.y=tmprect.y+7+32;
	
	SDL_FillRect(screen,&namerect,color.textentrybox);
	puttext(namerect.x+4,namerect.y+5,4,color.textentrytext,username);

	okayrect.x=tmprect.x+tmprect.w-32;
	okayrect.y=tmprect.y+7+64;
	okayrect.h=CHARHEIGHT*2+3;
	okayrect.w=CHARWIDTH*2*2+1;

	//SDL_FillRect(screen,&okayrect,color.textentrybox);
	puttext(okayrect.x+3,okayrect.y+3,2,color.normaltext,"OK");


	updateall();  // fix -- is this really needed? we should soil() the
	              // above rectangles instead, i  think


	SDL_EnableUNICODE(1);
	
	clean();
	do
	{
		SDL_WaitEvent(NULL); // no new CPU cooler needed. :)
		while (pollevent(&event))
		{
			if (event.type == SDL_QUIT)
			{
				done=true;
				quit=true;
			}
			else if (event.type == SDL_MOUSEBUTTONDOWN)
			{  // fix -- make this left button only
				if (event.button.x>=okayrect.x
				 && event.button.y>=okayrect.y
				 && event.button.x<okayrect.x+okayrect.w
				 && event.button.y<okayrect.y+okayrect.h)
				okaypressed=true;
			}
			else if (event.type == SDL_MOUSEBUTTONUP)
			{
				if (okaypressed
				 && event.button.x>=okayrect.x
				 && event.button.y>=okayrect.y
				 && event.button.x<okayrect.x+okayrect.w
				 && event.button.y<okayrect.y+okayrect.h)
				{
					if (username[0]!='\0')
					{
						done=true;
					}
					//FIX: else beep
				}
				else if (okayglow)
				{
					okayglow=false;
					SDL_FillRect(screen,&okayrect,color.background);
					puttext(okayrect.x+3,okayrect.y+3,2,color.normaltext,"OK");
					soil(okayrect);
				}
				okaypressed=false;
			}
			else if (event.type == SDL_MOUSEMOTION)
			{
				if (event.motion.x>=okayrect.x
				 && event.motion.y>=okayrect.y
				 && event.motion.x<okayrect.x+okayrect.w
				 && event.motion.y<okayrect.y+okayrect.h)
				 {
					if (!okayglow)
					{
						okayglow=true;
						SDL_FillRect(screen,&okayrect,color.menuhighlight);
						puttext(okayrect.x+3,okayrect.y+3,2,color.background,"OK");
						soil(okayrect);
					}
					
				}
				else
				{
					if (okayglow && !okaypressed)
					{
						okayglow=false;
						SDL_FillRect(screen,&okayrect,color.background);
						puttext(okayrect.x+3,okayrect.y+3,2,color.normaltext,"OK");
						soil(okayrect);
					}
				}
			
			}
			else if (event.type==SDL_KEYDOWN)
			{

				if (event.key.keysym.sym==SDLK_BACKSPACE ||
				    event.key.keysym.sym==SDLK_DELETE ||
				    event.key.keysym.sym==SDLK_LEFT)
				{
					if (clearname)
					{
						insertionpoint=0;
						username[0]='\0'; // null out username
					}
					else if (insertionpoint>0)
					{
						insertionpoint--;	
						username[insertionpoint]='\0';
					}
					//FIX: else beep
					clearname=false;
				}	
				else if (event.key.keysym.sym==SDLK_RETURN || event.key.keysym.sym==SDLK_KP_ENTER)
				{
					if (username[0]!='\0')
					{
						done=true;
					}
					//FIX: else beep
					clearname=false;
				}
				else if (event.key.keysym.sym==SDLK_ESCAPE || event.key.keysym.sym==SDLK_CLEAR) // fix -- make ctrl-u work here
				{
					insertionpoint=0;
					username[0]='\0'; // null out username
					clearname=false;
				}
				else if ((event.key.keysym.unicode & 0xFF80) == 0) // make sure it's ascii
				{
					keypressed=event.key.keysym.unicode & 0x7F;
					if (keypressed==32) keypressed=95;
					if (keypressed>32 && keypressed<127)
					{
						if (clearname)
						{
							insertionpoint=0;
							clearname=false;
						}
						if (insertionpoint<12)
						{
							username[insertionpoint]=keypressed;
							username[insertionpoint+1]='\0';
							insertionpoint++;
						}
						//FIX: else beep
					}			
				}
				SDL_FillRect(screen,&namerect,color.textentrybox);
				puttext(namerect.x+4,namerect.y+5,4,color.textentrytext,username);
				soil(namerect);	
			}
		}
		clean();
	} while (!done);

	// because the game is over, restore of background is unnecessary
	
	SDL_EnableUNICODE(0);	
	return(quit);
}

#define MAINMENULENGTH 5
int popuplevelmenu()
{
	SDL_Rect menurect, menubuttonrect;
	int rc;
	SDL_Surface * levelmenusave;

	char * mainmenu[MAINMENULENGTH] = { "NEW GAME", "OPTIONS", "HIGH SCORES", "HELP","QUIT" };
	int (*mainmenufunctions[MAINMENULENGTH])(char *) = { &menuitem_newgame, &menuitem_options, &menuitem_highscores, &menuitem_help, &menuitem_quit };
	
	menurect.w=128;
	menurect.h=7+(MAINMENULENGTH*(CHARHEIGHT*2+3));
	menurect.x=BORDERRIGHT-menurect.w+5;
	menurect.y=BORDERBOTTOM-menurect.h+9;
	
	menubuttonrect.x=WIDTH-(CHARWIDTH*2*4)-MARGINRIGHT-4;
	menubuttonrect.y=BOTTOMSTATUSY;
	menubuttonrect.w=CHARWIDTH*2*4+3;
	menubuttonrect.h=CHARHEIGHT*2+3;

	levelmenusave = SDL_CreateRGBSurface(SDL_SWSURFACE,menurect.w,menurect.h,screen->format->BitsPerPixel,0,0,0,0);
	SDL_BlitSurface(screen, &menurect, levelmenusave, NULL);

	rc=menuhandler(menurect, MAINMENULENGTH, mainmenu, mainmenufunctions, 0);


	if (rc != POPUPMENUQUITGAME)
	{
		if (gameflags.themechanged)
		{
			if (!strcmp(options.theme,"random") && rc != POPUPMENUNEWGAME)
				settheme("random");
	
			redrawwholelevel();
			drawmenubutton(&menubuttonrect,true);
			gameflags.themechanged=false;
			updateall();
		}
		else
		{
			SDL_BlitSurface(levelmenusave, NULL, screen, &menurect);
			soil(menurect);
		}
	}	
	
	SDL_FreeSurface(levelmenusave);	
	
	return rc;
}

int menuitem_newgame(char * notused)
{
	return POPUPMENUNEWGAME;
}


int menuitem_options(char * notused)
{
	SDL_Event event;
	
	int rc=popupoptionsmenu();

	if (rc!=POPUPMENUEXITMENU)
	{
		return rc;
	}
	else
	{
		// add a fake event so "Options" gets un-highlighted if need be.
		SDL_GetMouseState((int *)&event.motion.x,(int *)&event.motion.y);
		event.type = SDL_MOUSEMOTION;
		SDL_PushEvent(&event);
		return POPUPMENUDONOTHING;
	}
}


int menuitem_highscores(char * notused)
{
	int rc;
	SDL_Event event;
	rc=popuphighscores();
	// add a fake event so menuitem gets un-highlighted if need be.
	SDL_GetMouseState((int *)&event.motion.x,(int *)&event.motion.y);
	event.type = SDL_MOUSEMOTION;
	SDL_PushEvent(&event);
	return rc;
}


int menuitem_help(char * notused)
{
	int rc;
	SDL_Event event;
	rc=popuphelp();
	// add a fake event so menuitem gets un-highlighted if need be.
	SDL_GetMouseState((int *)&event.motion.x,(int *)&event.motion.y);
	event.type = SDL_MOUSEMOTION;
	SDL_PushEvent(&event);

	return rc;
}


int menuitem_quit(char * notused)
{
	return POPUPMENUQUITGAME;
}

#define OPTIONSMENULENGTH 5

int popupoptionsmenu()
{
	SDL_Rect menurect;
	GameDifficultyType originaldifficulty=options.difficulty;
	char originaltheme[MAXTHEMENAMELENGTH+1];
	int rc;
	int originalthemecl=false;
	int recallme=false;


	char * optionsmenu[OPTIONSMENULENGTH] = { "SOUND", "AUTO PAUSE", "FULL SCREEN", "DIFFICULTY", "THEME" };
	int (*optionsmenufunctions[OPTIONSMENULENGTH])(char *) = { &menuitem_sound, &menuitem_autopause, &menuitem_fullscreen, &menuitem_difficulty, &menuitem_theme };

	if (strlen(commandline.theme)>0)
	{
		snprintf(originaltheme,MAXTHEMENAMELENGTH+1,"%s",commandline.theme);
		originalthemecl=true;
	}
	else
	{
		snprintf(originaltheme,MAXTHEMENAMELENGTH+1,"%s",options.theme);
		originalthemecl=false;	
	}
				
	menurect.w=229;
	menurect.h=7+(OPTIONSMENULENGTH*(CHARHEIGHT*2+3));
	menurect.x=BORDERRIGHT-menurect.w+10;
	menurect.y=BORDERBOTTOM-menurect.h+9-((CHARHEIGHT*2+4)*4)-2;

	optionsmenusave = SDL_CreateRGBSurface(SDL_SWSURFACE,menurect.w,menurect.h,screen->format->BitsPerPixel,0,0,0,0);
	SDL_BlitSurface(screen, &menurect, optionsmenusave, NULL);

	rc=menuhandler(menurect, OPTIONSMENULENGTH, optionsmenu, optionsmenufunctions, 84);

	if (rc==POPUPMENUEXITMENU)
	{
		if ((originalthemecl && strcmp(commandline.theme, originaltheme)) ||
		    (!originalthemecl && strcmp(options.theme, originaltheme)) )
		{
			gameflags.themechanged=true;
		}
	}

	if (rc==POPUPMENUEXITMENU && options.difficulty != originaldifficulty)
	{
		if (checkiflevelatstart())
		{
			rc=POPUPMENUNEWGAME;
		}
		else
		{
			switch (yesnodialog("Changing difficulty","requires restart.","End this game?"))
			{
				case YESNODIALOGQUITGAME:
					rc=POPUPMENUQUITGAME;
				break;
				case YESNODIALOGYES:
					rc=POPUPMENUNEWGAME;
				break;
				case YESNODIALOGNO:
					options.difficulty=originaldifficulty; // put things back. :)
					recallme=true;
				break;
			}
		}
	}

	//writeoptions(); // probably no need to do this until exit
	
	// and restore the background
	if (rc != POPUPMENUQUITGAME)
	{
		SDL_BlitSurface(optionsmenusave, NULL, screen, &menurect);
		soil(menurect);
	}

	SDL_FreeSurface(optionsmenusave);	
	

	if (recallme) rc=popupoptionsmenu(); // this is a bit sick.	
	return rc;
}


int menuitem_sound(char * val)
{
	if (strlen(val)==0)
	{
		if (gameflags.soundsystemworks)
		{
			if (options.sound==SOUNDON)
				strncpy(val,"on",MAXMENUVALUELENGTH);
			else
				strncpy(val,"off",MAXMENUVALUELENGTH);
		}
		else
		{
			strncpy(val,"n/a",MAXMENUVALUELENGTH);
		}
		return POPUPMENUDONOTHING;
	}
	else
	{
		if (!gameflags.soundsystemworks) return POPUPMENUDONOTHING;
		if (options.sound==SOUNDON)
		{
			options.sound=SOUNDOFF;
			strncpy(val,"off",MAXMENUVALUELENGTH);
		}
		else
		{
			options.sound=SOUNDON;	
			strncpy(val,"on",MAXMENUVALUELENGTH);
		}
		return POPUPMENUREDRAWME;
	}
}

int menuitem_autopause(char * val)
{
	if (strlen(val)==0)
	{
		if (options.autopause==AUTOPAUSEON)
			strncpy(val,"on",MAXMENUVALUELENGTH);
		else
			strncpy(val,"off",MAXMENUVALUELENGTH);
		return POPUPMENUDONOTHING;
	}
	else
	{
		if (options.autopause==AUTOPAUSEON)
		{
			options.autopause=AUTOPAUSEOFF;
			strncpy(val,"off",MAXMENUVALUELENGTH);
		}
		else
		{
			options.autopause=AUTOPAUSEON;	
			strncpy(val,"on",MAXMENUVALUELENGTH);
		}
		return POPUPMENUREDRAWME;
	}
}

int menuitem_fullscreen(char * val)
{
	// fix -- cope with failure to switch
	if (strlen(val)==0)
	{
		switch (options.fullscreen)
		{
			case FULLSCREENOFF:
				strncpy(val,"off",MAXMENUVALUELENGTH);
			break;
			case FULLSCREENON:
				strncpy(val,"on",MAXMENUVALUELENGTH);
			break;
			case FULLSCREENALWAYS:
				strncpy(val,"always",MAXMENUVALUELENGTH);
			break;
			case FULLSCREENUNKNOWN: // this should never happen!
				strncpy(val,"???",MAXMENUVALUELENGTH);
			break;
		}
		return POPUPMENUDONOTHING;
	}
	else
	{
		switch (options.fullscreen)
		{
			case FULLSCREENOFF:
				options.fullscreen=FULLSCREENON;
				strncpy(val,"on",MAXMENUVALUELENGTH);
				makefullscreen();
			break;
			case FULLSCREENON:
				options.fullscreen=FULLSCREENALWAYS;
				strncpy(val,"always",MAXMENUVALUELENGTH);
			break;
			case FULLSCREENALWAYS:
				options.fullscreen=FULLSCREENOFF;
				strncpy(val,"off",MAXMENUVALUELENGTH);
				makewindowed();
			break;
			case FULLSCREENUNKNOWN: // this should never happen!
				options.fullscreen=FULLSCREENOFF;
				strncpy(val,"off",MAXMENUVALUELENGTH);
				makewindowed();
			break;
		}
		return POPUPMENUREDRAWME;
	}
}


int menuitem_difficulty(char * val)
{
	if (strlen(val)==0)
	{
		switch (options.difficulty)
		{
			case NORMAL:
				strncpy(val,"normal",MAXMENUVALUELENGTH);
			break;
			case EASY:
				strncpy(val,"easy",MAXMENUVALUELENGTH);
			break;
			case HARD:
				strncpy(val,"hard",MAXMENUVALUELENGTH);
			break;
		}
		return POPUPMENUDONOTHING;
	}
	else
	{
		switch (options.difficulty)
		{
			case NORMAL:
				options.difficulty=EASY;
				strncpy(val,"easy",MAXMENUVALUELENGTH);
			break;
			case EASY:
				options.difficulty=HARD;
				strncpy(val,"hard",MAXMENUVALUELENGTH);
			break;
			case HARD:
				options.difficulty=NORMAL;
				strncpy(val,"normal",MAXMENUVALUELENGTH);
			break;
		}
		return POPUPMENUREDRAWME;
	}
}

int menuitem_theme(char * val)
{
	char** themelist;
	int t;
	int themecount;
	SDL_Rect tmprect;
	
	// FIX -- this is kind of kludgy -- why define this twice?
	char * mainmenu[MAINMENULENGTH] = { "NEW GAME", "OPTIONS", "HIGH SCORES", "HELP","QUIT" };	

	// fix -- all of these are also stupidly duplicated
	SDL_Rect mainmenurect;
	SDL_Rect optionsmenurect;
	SDL_Rect menubuttonrect;
	mainmenurect.w=128;
	mainmenurect.h=7+(MAINMENULENGTH*(CHARHEIGHT*2+3));
	mainmenurect.x=BORDERRIGHT-mainmenurect.w+5;
	mainmenurect.y=BORDERBOTTOM-mainmenurect.h+9;

	optionsmenurect.w=229;
	optionsmenurect.h=7+(OPTIONSMENULENGTH*(CHARHEIGHT*2+3));
	optionsmenurect.x=BORDERRIGHT-optionsmenurect.w+10;
	optionsmenurect.y=BORDERBOTTOM-optionsmenurect.h+9-((CHARHEIGHT*2+4)*4)-2;

	menubuttonrect.x=WIDTH-(CHARWIDTH*2*4)-MARGINRIGHT-4;
	menubuttonrect.y=BOTTOMSTATUSY;
	menubuttonrect.w=CHARWIDTH*2*4+3;
	menubuttonrect.h=CHARHEIGHT*2+3;

                                        


	if (strlen(val)==0)
	{
		if (strlen(commandline.theme)>0)
			strncpy(val,commandline.theme,MAXMENUVALUELENGTH);
		else
			strncpy(val,options.theme,MAXMENUVALUELENGTH);
		return POPUPMENUDONOTHING;
	}
	else
	{
		themecount=getthemenames(&themelist);
		
		if (strlen(commandline.theme)>0)
		{
			t=getthemenumber(themelist,themecount,commandline.theme);
			// once we've changed the theme, there's no going back
			*(commandline.theme)='\0'; // makes this ""
		}
		else
		{
			t=getthemenumber(themelist,themecount,options.theme);
		}

		if (t==themecount-1)
		{ // "random", at the end of the list
			
			strncpy(val,"random",MAXMENUVALUELENGTH);
			snprintf(options.theme,MAXMENUVALUELENGTH,"random");
			
			settheme("linux"); // just for pretty
			
			redrawwholelevel();
			drawmenu(mainmenurect, MAINMENULENGTH, mainmenu, NULL, 0, 1);
			drawmenubutton(&menubuttonrect,true);
			
			puttext((WIDTH-gettextwidth(10,"?"))/2,(HEIGHT-CHARHEIGHT*10)/2,10,color.gameovertext,"?");
			gameflags.themechanged=true;
		}
		else
		{
			strncpy(val,themelist[(t+1)%themecount],MAXMENUVALUELENGTH);
			snprintf(options.theme,MAXMENUVALUELENGTH,themelist[(t+1)%themecount]);
			
			settheme(options.theme);
			
			redrawwholelevel();
			drawmenu(mainmenurect, MAINMENULENGTH, mainmenu, NULL, 0, 1);
			drawmenubutton(&menubuttonrect,true);
		}
		SDL_BlitSurface(screen, &optionsmenurect, optionsmenusave, NULL);
		
		freethemenames(&themelist,themecount);
		
		tmprect.w=WIDTH; tmprect.h=HEIGHT;
		tmprect.x=0;tmprect.y=0;
		soil(tmprect);
		
		
		return POPUPMENUDONOTHING;
	}
}


int popuphighscores()
{
	SDL_Rect scorelistrect,borderrect;
	SDL_Surface * scorelistsave;
	int quit=POPUPMENUDONOTHING;
	int i;
	char buf[30]; // plenty big. :)

	scorelistrect.w=PLAYWIDTH-(BLOCKWIDTH*11);
	scorelistrect.h=PLAYHEIGHT-(BLOCKHEIGHT*4);
	scorelistrect.x=BORDERLEFT+PLAYWIDTH/2-scorelistrect.w/2+BLOCKWIDTH*4;
	scorelistrect.y=BORDERTOP+PLAYHEIGHT/2-scorelistrect.h/2+BLOCKHEIGHT/2;
	
	borderrect.w=scorelistrect.w+2;
	borderrect.h=scorelistrect.h+2;
	borderrect.x=scorelistrect.x-1;
	borderrect.y=scorelistrect.y-1;

	scorelistsave = SDL_CreateRGBSurface(SDL_SWSURFACE,borderrect.w,borderrect.h,screen->format->BitsPerPixel,0,0,0,0);
	SDL_BlitSurface(screen, &borderrect, scorelistsave, NULL);

	SDL_FillRect(screen,&borderrect,color.gridline);
	SDL_FillRect(screen,&scorelistrect,color.background);

	puttext(scorelistrect.x+(scorelistrect.w/2-(CHARWIDTH*2*9)),scorelistrect.y+BLOCKHEIGHT,4,color.bonusscrolltext,"HIGH SCORES");

	// just in case they've changed...
	readhiscores();

	for (i=0;i<HISCORENUM;i++)
	{
		snprintf(buf,4,"%d.",i+1);
		puttext(scorelistrect.x+BLOCKWIDTH,scorelistrect.y+45+i*(CHARHEIGHT*2+5),2,color.normaltext,buf);		
		puttext(scorelistrect.x+BLOCKWIDTH*4,scorelistrect.y+45+i*(CHARHEIGHT*2+5),2,color.normaltext,hiscorename[i]);		
		snprintf(buf,30,"%ld",hiscoreval[i]);		
		puttext(scorelistrect.x+scorelistrect.w-(BLOCKWIDTH*5),scorelistrect.y+45+i*(CHARHEIGHT*2+5),2,color.normaltext,buf);
	}
	
	soil(borderrect);	

	clean();
		
	if (waitforuser()) quit=POPUPMENUQUITGAME;

	SDL_BlitSurface(scorelistsave, NULL, screen, &borderrect);
	soil(borderrect);
	clean();

	SDL_FreeSurface(scorelistsave);	 	

	return(quit);
}

int popuphelp()
{
	SDL_Rect helprect,borderrect;
	SDL_Surface * helpsave;
	int quit=POPUPMENUDONOTHING;
	int i=0;
	char buf[80];


#ifdef ONEBUTTONMICEARESILLY
	char helptext[13][50]={
	     "Ok, so there's a bunch of penguins on an",
	     "iceberg in Antarctica. You must catch them",
	     "so they can be shipped to Finland. The",
	     "smaller the area in which they're caught,",
	     "the lower the shipping fees and the higher",
	     "your score - but don't take too long: the",
	     "clock is ticking. Once 80% of the 'berg is",
	     "cleared, it's on to the next shipment.",
	     "",
	     "The left mouse button starts lines; right",
	     "button or [spacebar] toggles direction.",
	     "",
	     "Check the README file for more info."
	      };
#else
	char helptext[13][50]={
	     "Ok, so there's a bunch of penguins on an",
	     "iceberg in Antarctica. You must catch them",
	     "so they can be shipped to Finland. The",
	     "smaller the area in which they're caught,",
	     "the lower the shipping fees and the higher",
	     "your score - but don't take too long: the",
	     "clock is ticking. Once 80% of the 'berg is",
	     "cleared, it's on to the next shipment.",
	     "",
	     "The left mouse button starts lines; right",
	     "button toggles direction. You'll catch on.",
	     "",
	     "Check the README file for more info."
	      };
#endif

	helprect.w=PLAYWIDTH-(BLOCKWIDTH*2)+2;
	helprect.h=PLAYHEIGHT-(BLOCKHEIGHT*3);
	helprect.x=BORDERLEFT+PLAYWIDTH/2-helprect.w/2;
	helprect.y=BORDERTOP+PLAYHEIGHT/2-helprect.h/2;
	
	borderrect.w=helprect.w+2;
	borderrect.h=helprect.h+2;
	borderrect.x=helprect.x-1;
	borderrect.y=helprect.y-1;

	helpsave = SDL_CreateRGBSurface(SDL_SWSURFACE,borderrect.w,borderrect.h,screen->format->BitsPerPixel,0,0,0,0);
	SDL_BlitSurface(screen, &borderrect, helpsave, NULL);

	SDL_FillRect(screen,&borderrect,color.gridline);
	SDL_FillRect(screen,&helprect,color.background);
	
	for (i=0;i<13;i++)
	{
		puttext(helprect.x+BLOCKWIDTH/2,helprect.y+BLOCKHEIGHT/2+i*(CHARHEIGHT*2+4),2,color.normaltext,helptext[i]);		
	}
	snprintf(buf,80,"v%d.%d.%d   %s",VERMAJOR,VERMINOR,VERSUB,"Copyright (c) 2000-2002 Matthew Miller. Released under the GPL.");
	puttext(helprect.x+BLOCKWIDTH/2,helprect.y+helprect.h-CHARHEIGHT*3,1,color.copyrighttext,buf);		
	puttext(helprect.x+BLOCKWIDTH/2,helprect.y+helprect.h-CHARHEIGHT*1-3,1,color.copyrighttext,"Thanks to my wonderful wife Karen for inspiration (and for patience)!");		
	
	soil(borderrect);	

	clean();

	if (waitforuser()) quit=POPUPMENUQUITGAME;

	SDL_BlitSurface(helpsave, NULL, screen, &borderrect);
	soil(borderrect);
	clean();

	SDL_FreeSurface(helpsave);	 	

	return(quit);
}


/* Wait for the user to hit a key or button or quit; returns true if quit */
int waitforuser()
{
	int done=false;
	int quit=false;
	SDL_Event event;
	
	do
	{
		SDL_WaitEvent(NULL);
		while(pollevent(&event))
		{
			if ( event.type == SDL_QUIT)
			{
				done=true;
				quit=true;
			}
			else if (event.type == SDL_MOUSEBUTTONUP)
			{
				done=true;
			}
			else if (event.type == SDL_KEYUP)
			{
				switch(translatekeyevent(&event))
				{
					case KEYCANCEL:     // falls through
					case KEYSWITCHLINE: // falls through
					case KEYSTARTLINE:  
						done=true;
					default:
					break;
				}
	
			}
		}
	} while (!done);
	return(quit);		
}




/* displays a menu; takes a rect for position and size, the length of the
 * menu, an array of menu items, an array of pointers to functions for
 * each menu item (functions should return type MenuReturnType
 * indicating what to do when the item is clicked) and a flag telling whether
 * menu items are option/value pairs.
 */
int menuhandler(SDL_Rect menurect, int menulength, char ** menuitems, int (**menufunctions)(char *), int menuvaluetextwidth)
{
	SDL_Rect insiderect;
	SDL_Rect menuitemrect[MAXMENUITEMS];
	char menuvalues[MAXMENUITEMS][MAXMENUVALUELENGTH];
	int menuitemglow=-1;
	int menuitempressed=false;
	SDL_Event event;
	int quit=POPUPMENUDONOTHING;
	int forceredraw=false;

	int i;
	
	insiderect.w=menurect.w-2;
	insiderect.h=menurect.h-2;
	insiderect.x=menurect.x+1;
	insiderect.y=menurect.y+1;

	// fix -- use drawmenu() here?
	SDL_FillRect(screen,&menurect,color.gridline);
	SDL_FillRect(screen,&insiderect,color.background);

	for (i=0;i<menulength;i++)
	{
		menuitemrect[i].x=insiderect.x;
		menuitemrect[i].y=insiderect.y+2+(i*(CHARHEIGHT*2+3));
		menuitemrect[i].w=insiderect.w;
		menuitemrect[i].h=(CHARHEIGHT*2)+3;
		puttext(menuitemrect[i].x+5,menuitemrect[i].y+3,2,color.normaltext,menuitems[i]);

		strncpy(menuvalues[i],"",MAXMENUVALUELENGTH);
		if (menuvaluetextwidth)
		{
			if (menufunctions!=NULL && menufunctions[i]!=NULL)
				(*menufunctions[i])(menuvalues[i]);
			puttext(menuitemrect[i].x+menuitemrect[i].w-menuvaluetextwidth,menuitemrect[i].y+3,2,color.normaltext,menuvalues[i]);
		}
	}		
	
	soil(menurect);	
	
	clean();
	
	
	// find options
	
	do
	{
		SDL_WaitEvent(NULL); // no new CPU cooler needed. :)
		while (pollevent(&event))
		{
			if (event.type == SDL_QUIT)
			{
				quit=POPUPMENUQUITGAME;
			}
			else if (event.type == SDL_MOUSEBUTTONDOWN)
			{  // fix -- make this left button only
				if (event.button.x>insiderect.x &&
				    event.button.y>insiderect.y &&
				    event.button.x<insiderect.x + insiderect.w &&
				    event.button.y<insiderect.y + insiderect.h)
				{
					menuitempressed=true;
				}
				else
				{
					quit=POPUPMENUEXITMENU;
				}
			}
			else if (event.type == SDL_MOUSEBUTTONUP)
			{
				if (menuitempressed)
				{
					// in area, button was down
					if (event.motion.x>insiderect.x &&
					    event.motion.y>(insiderect.y + 2) &&
					    event.motion.x<insiderect.x + insiderect.w &&
					    event.motion.y<insiderect.y + insiderect.h - 2)
					{
						if (menufunctions!=NULL && menufunctions[menuitemglow]!=NULL)
							quit=(*menufunctions[menuitemglow])(menuvalues[menuitemglow]);
							
						// add a fake event so "Options" gets un-highlighted if need be.
						// fix -- redraw shouldn't be tied to mouse motion
						SDL_GetMouseState((int *)&event.motion.x,(int *)&event.motion.y);
						event.type = SDL_MOUSEMOTION;
						SDL_PushEvent(&event);
						forceredraw=true;
						
						// need for full repaint
						SDL_FillRect(screen,&menurect,color.gridline);
						SDL_FillRect(screen,&insiderect,color.background);
						soil(menurect);

					}					
					menuitempressed=false;
				}
			}
			else if (event.type == SDL_MOUSEMOTION)
			{
				// are we in the menu area?
				if (forceredraw || 
				    (event.motion.x>insiderect.x &&
				     event.motion.y>(insiderect.y + 2) &&
				     event.motion.x<insiderect.x + insiderect.w &&
				     event.motion.y<insiderect.y + insiderect.h - 2))
				{
					forceredraw=false;
					// FIX -- should check for overflow, just to be sure.
					menuitemglow=(event.motion.y-(insiderect.y+3))/(CHARHEIGHT*2+3);
					// FIX -- this is inefficient.
					// fix -- redraw shouldn't be tied to mouse motion
					SDL_FillRect(screen,&insiderect,color.background);
					for (i=0;i<menulength;i++)
					{
						if (i == menuitemglow)
						{
							SDL_FillRect(screen,&menuitemrect[i],color.menuhighlight);
							puttext(menuitemrect[i].x+5,menuitemrect[i].y+3,2,color.background,menuitems[i]);
							if (menuvaluetextwidth) puttext(menuitemrect[i].x+menuitemrect[i].w-menuvaluetextwidth,menuitemrect[i].y+3,2,color.background,menuvalues[i]);
						}
						else
						{
							puttext(menuitemrect[i].x+5,menuitemrect[i].y+3,2,color.normaltext,menuitems[i]);
							if (menuvaluetextwidth) puttext(menuitemrect[i].x+menuitemrect[i].w-menuvaluetextwidth,menuitemrect[i].y+3,2,color.normaltext,menuvalues[i]);
						}
					}		
					soil(insiderect);
				}
				else
				{
					if (menuitemglow != -1 && !menuitempressed)
					{
						menuitemglow=-1;
						SDL_FillRect(screen,&insiderect,color.background);
						for (i=0;i<menulength;i++)
						{
							puttext(menuitemrect[i].x+5,menuitemrect[i].y+3,2,color.normaltext,menuitems[i]);
							if (menuvaluetextwidth) puttext(menuitemrect[i].x+menuitemrect[i].w-menuvaluetextwidth,menuitemrect[i].y+3,2,color.normaltext,menuvalues[i]);
						}
						soil(insiderect);
					}
				}
			
			}
			else if (event.type==SDL_KEYUP)
			{ // fix -- add support for movement keys, and for select keys.
				switch(translatekeyevent(&event))
				{
					case KEYCANCEL:
						quit=POPUPMENUEXITMENU;
					default:
					break;
				}

			}
		}
		
		if (quit==POPUPMENUREDRAWME)
		{	
			quit=POPUPMENUDONOTHING;
	
			// add a fake event so "Options" gets un-highlighted if need be.
			SDL_GetMouseState((int *)&event.motion.x,(int *)&event.motion.y);
			event.type = SDL_MOUSEMOTION;
			SDL_PushEvent(&event);
			forceredraw=true;
	
			// need for full repaint
			SDL_FillRect(screen,&menurect,color.gridline);
			SDL_FillRect(screen,&insiderect,color.background);
			soil(menurect);
		}
		
		clean();
	} while (quit==POPUPMENUDONOTHING);

	// a fake event to update things that depend on mouse position
	SDL_GetMouseState((int *)&event.motion.x,(int *)&event.motion.y);
	event.type = SDL_MOUSEMOTION;
	SDL_PushEvent(&event);

	return(quit);

}

void drawmenu(SDL_Rect menurect, int menulength, char ** menuitems, int (**menufunctions)(char *), int menuvaluetextwidth, int highlighted)
{
	SDL_Rect insiderect;
	SDL_Rect menuitemrect[MAXMENUITEMS];
	char menuvalues[MAXMENUITEMS][MAXMENUVALUELENGTH];

	int i;
	
	insiderect.w=menurect.w-2;
	insiderect.h=menurect.h-2;
	insiderect.x=menurect.x+1;
	insiderect.y=menurect.y+1;

	SDL_FillRect(screen,&menurect,color.gridline);
	SDL_FillRect(screen,&insiderect,color.background);

	for (i=0;i<menulength;i++)
	{
		menuitemrect[i].x=insiderect.x;
		menuitemrect[i].y=insiderect.y+2+(i*(CHARHEIGHT*2+3));
		menuitemrect[i].w=insiderect.w;
		menuitemrect[i].h=(CHARHEIGHT*2)+3;

		strncpy(menuvalues[i],"",MAXMENUVALUELENGTH);
		if (i == highlighted)
		{
			SDL_FillRect(screen,&menuitemrect[i],color.menuhighlight);
			puttext(menuitemrect[i].x+5,menuitemrect[i].y+3,2,color.background,menuitems[i]);
			if (menuvaluetextwidth)
			{
				if (menufunctions!=NULL && menufunctions[i]!=NULL)
					(*menufunctions[i])(menuvalues[i]);
				puttext(menuitemrect[i].x+menuitemrect[i].w-menuvaluetextwidth,menuitemrect[i].y+3,2,color.background,menuvalues[i]);
			}			
		}
		else
		{
			puttext(menuitemrect[i].x+5,menuitemrect[i].y+3,2,color.normaltext,menuitems[i]);
			if (menuvaluetextwidth)
			{
				if (menufunctions!=NULL && menufunctions[i]!=NULL)
					(*menufunctions[i])(menuvalues[i]);
				puttext(menuitemrect[i].x+menuitemrect[i].w-menuvaluetextwidth,menuitemrect[i].y+3,2,color.normaltext,menuvalues[i]);
		 	}
		}

	}		
	
	soil(menurect);	
	
	clean();
	
}

void drawmenubutton(SDL_Rect* mbrect,int highlighted)
{
	if (highlighted)
	{
		SDL_FillRect(screen,mbrect,color.menuhighlight);
		puttext(mbrect->x+3,mbrect->y+3,2,color.background,"MENU"); 
	}
	else
	{
		SDL_FillRect(screen,mbrect,color.background);
		puttext(mbrect->x+3,mbrect->y+3,2,color.normaltext,"MENU");
	}
	soil(*mbrect);
}

// fix -- make this more generic (one or two buttons, selectable button text)
int yesnodialog(char* text1, char* text2, char* text3)
{
	SDL_Rect tmprect, yesrect,norect;
	SDL_Event event;
	int done=false;
	int rc=0;
	int yesglow=false;
	int yespressed=false;
	int noglow=false;
	int nopressed=false;
	SDL_Surface* bgsave;
	
	tmprect.w=gettextwidth(2,text1);
	if (gettextwidth(2,text2)>tmprect.w) tmprect.w=gettextwidth(2,text2);
	if (gettextwidth(2,text3)>tmprect.w) tmprect.w=gettextwidth(2,text3);
	tmprect.w+=15;
	tmprect.h=92; // fix -- make dialog shorter if text3=null. maybe also text2, but that might be too short
	tmprect.x=(WIDTH/2)-(tmprect.w/2);
	tmprect.y=(HEIGHT/2)-(tmprect.h/2);
	
	// fix -- error checking if this fails
	bgsave = SDL_CreateRGBSurface(SDL_SWSURFACE,tmprect.w,tmprect.h,screen->format->BitsPerPixel,0,0,0,0);
	SDL_BlitSurface(screen, &tmprect, bgsave, NULL);

	SDL_FillRect(screen,&tmprect,color.gridline);
	
	tmprect.w-=2; tmprect.h-=2;
	tmprect.x++; tmprect.y++;
	SDL_FillRect(screen,&tmprect,color.background);
	tmprect.w+=2; tmprect.h+=2; // yay kludgy
	tmprect.x--; tmprect.y--;
	
	

	puttext(tmprect.x+(tmprect.w-gettextwidth(2,text1))/2,tmprect.y+7,2,color.normaltext,text1);
	puttext(tmprect.x+(tmprect.w-gettextwidth(2,text2))/2,tmprect.y+7+CHARHEIGHT*3,2,color.normaltext,text2);
	puttext(tmprect.x+(tmprect.w-gettextwidth(2,text3))/2,tmprect.y+7+CHARHEIGHT*6,2,color.normaltext,text3);
	
	norect.x=tmprect.x+tmprect.w-33;
	norect.y=tmprect.y+7+64;
	norect.h=CHARHEIGHT*2+3;
	norect.w=CHARWIDTH*2*2+1;
	
	yesrect.x=tmprect.x+7;
	yesrect.y=tmprect.y+7+64;
	yesrect.h=CHARHEIGHT*2+3;
	yesrect.w=CHARWIDTH*2*3+1;

	puttext(norect.x+3,norect.y+3,2,color.normaltext,"NO");
	puttext(yesrect.x+3,yesrect.y+3,2,color.normaltext,"YES");

	
	soil(tmprect);
	
	clean();
	do
	{
		SDL_WaitEvent(NULL);
		while (pollevent(&event))
		{
			if (event.type == SDL_QUIT)
			{
				done=true;
				rc=YESNODIALOGQUITGAME;
			}
			else if (event.type == SDL_MOUSEBUTTONDOWN)
			{  // fix -- make this left button only
				if (event.button.x>=norect.x
				 && event.button.y>=norect.y
				 && event.button.x<norect.x+norect.w
				 && event.button.y<norect.y+norect.h)
				nopressed=true;
				else if (event.button.x>=yesrect.x
				 && event.button.y>=yesrect.y
				 && event.button.x<yesrect.x+yesrect.w
				 && event.button.y<yesrect.y+yesrect.h)
				yespressed=true;
			}
			else if (event.type == SDL_MOUSEBUTTONUP)
			{
				if (nopressed
				 && event.button.x>=norect.x
				 && event.button.y>=norect.y
				 && event.button.x<norect.x+norect.w
				 && event.button.y<norect.y+norect.h)
				{
					done=true;
					rc=YESNODIALOGNO;
				}
				else if (noglow)
				{
					noglow=false;
					SDL_FillRect(screen,&norect,color.background);
					puttext(norect.x+3,norect.y+3,2,color.normaltext,"NO");
					soil(norect);
				}
				nopressed=false;
				
				if (yespressed
				 && event.button.x>=yesrect.x
				 && event.button.y>=yesrect.y
				 && event.button.x<yesrect.x+yesrect.w
				 && event.button.y<yesrect.y+yesrect.h)
				{
					done=true;
					rc=YESNODIALOGYES;
				}
				else if (yesglow)
				{
					yesglow=false;
					SDL_FillRect(screen,&yesrect,color.background);
					puttext(yesrect.x+3,yesrect.y+3,2,color.normaltext,"YES");
					soil(yesrect);
				}
				yespressed=false;
			}
			else if (event.type == SDL_MOUSEMOTION)
			{
				if (event.motion.x>=norect.x
				 && event.motion.y>=norect.y
				 && event.motion.x<norect.x+norect.w
				 && event.motion.y<norect.y+norect.h)
				 {
					if (!noglow)
					{
						noglow=true;
						SDL_FillRect(screen,&norect,color.menuhighlight);
						puttext(norect.x+3,norect.y+3,2,color.background,"NO");
						soil(norect);
					}
					
				}
				else
				{
					if (noglow && !nopressed)
					{
						noglow=false;
						SDL_FillRect(screen,&norect,color.background);
						puttext(norect.x+3,norect.y+3,2,color.normaltext,"NO");
						soil(norect);
					}
				}
				if (event.motion.x>=yesrect.x
				 && event.motion.y>=yesrect.y
				 && event.motion.x<yesrect.x+yesrect.w
				 && event.motion.y<yesrect.y+yesrect.h)
				 {
					if (!yesglow)
					{
						yesglow=true;
						SDL_FillRect(screen,&yesrect,color.menuhighlight);
						puttext(yesrect.x+3,yesrect.y+3,2,color.background,"YES");
						soil(yesrect);
					}
					
				}
				else
				{
					if (yesglow && !yespressed)
					{
						yesglow=false;
						SDL_FillRect(screen,&yesrect,color.background);
						puttext(yesrect.x+3,yesrect.y+3,2,color.normaltext,"YES");
						soil(yesrect);
					}
				}
			
			}
			else if (event.type==SDL_KEYDOWN)
			{
				if (event.key.keysym.sym==SDLK_n)
				{
					done=true;
					rc=YESNODIALOGNO;
				}	
				else if (event.key.keysym.sym==SDLK_ESCAPE)
				{
					// little hack to keep escape key from "bleeding" into
					// cancelling the whole menu
					done=true;
					rc=YESNODIALOGNO;
					SDL_Delay(200);
					while(pollevent(&event) && (event.type==SDL_KEYDOWN || event.type==SDL_KEYUP));
				}
				else if (event.key.keysym.sym==SDLK_y || event.key.keysym.sym==SDLK_RETURN || event.key.keysym.sym==SDLK_KP_ENTER)
				{
					done=true;
					rc=YESNODIALOGYES;
				}
			}
		}
		clean();
	} while (!done);

	SDL_BlitSurface(bgsave, NULL, screen, &tmprect);
	soil(tmprect);
	clean();
	        
	return(rc);
}
